#!/usr/bin/perl
#
# $Revision: 1.1 $
# Copyright 2013 Teleflora
#
# Uninstall the offcycle edir update package.
#

use strict;
use warnings;
use Getopt::Long;
use POSIX;
use English qw( -no_match_vars );
use File::Basename;

our $VERSION = '1.1';

my $CVS_REVISION = '$Revision: 1.1 $';
my $PROGNAME = basename($PROGRAM_NAME);

#######################################
#########   D E F A U L T S   #########
#######################################

my $DAISY_FILESYS_DEF              = "/d";
my $DAISY_DIR_DEF                  = $DAISY_FILESYS_DEF . "/daisy";
my $DAISY_BINDIR_DEF               = $DAISY_DIR_DEF     . "/bin";
my $DAISY_CONFDIR_DEF              = $DAISY_DIR_DEF     . "/config";
my $EDIR_BASEDIR_DEF               = $DAISY_FILESYS_DEF . "/edirectories";
my $EDIR_INSTALLBASE_SCRIPT_DEF    = $DAISY_BINDIR_DEF  . "/edir_installbase.pl";
my $EDIR_UPDATE_SCRIPT_DEF         = $DAISY_BINDIR_DEF  . "/edir_update.pl";
my $EDIR_REVERT_SCRIPT_DEF         = $DAISY_BINDIR_DEF  . "/edir_revert.pl";
my $EDIR_UNINSTALL_LOGFILE_DEF     = $DAISY_DIR_DEF     . "/log/edir_uninstall.log";;
my $EDIR_INSTALLBASE_LOGFILE_DEF   = $DAISY_DIR_DEF     . "/log/edir_installbase.log";
my $EDIR_UPDATE_LOGFILE_DEF        = $DAISY_DIR_DEF     . "/log/edir_update.log";
my $EDIR_REVERT_LOGFILE_DEF        = $DAISY_DIR_DEF     . "/log/edir_revert.log";
my $EDIR_SUMMARY_LOGFILE_DEF       = $DAISY_DIR_DEF     . "/log/Delta_Stats.log";
my $EDIR_UPDATE_CONF_DEF           = $DAISY_DIR_DEF     . "/config/edir_update.conf";
my $EDIR_LOCKFILE_DEF              = $DAISY_DIR_DEF     . "/config/edir_update.lock";
my $EDIR_PIDFILE_DEF               = $DAISY_DIR_DEF     . "/config/edir_update.pid";
my $EDIR_BOOK_DEF                  = "jan";
my $EDIR_YEAR_DEF                  = "2013";
my $EDIR_CRON_JOB_FILE_DEF         = "/etc/cron.d/edir_update";

############################################
#########   E X I T  S T A T U S   #########
############################################

my $EXIT_OK                        = 0;
my $EXIT_COMMAND_LINE              = 1;
my $EXIT_MUST_BE_ROOT              = 2;

#####################################
#########   G L O B A L S   #########
#####################################

my $EMPTY                          = q{};
my $DAISY                          = 1;
my $DAISY_DIR                      = $DAISY_DIR_DEF;
my $LOGFILE                        = $EDIR_UNINSTALL_LOGFILE_DEF;
my $LOG_STDERR                     = 0;
my $DEBUG                          = 0;
my $TIMESTAMP                      = strftime("%Y%m%d%H%M%S", localtime());
my $ALTROOT                        = $EMPTY;   # alternate root for testing
my $ALTOS                          = $EMPTY;     # alternate os for testing

##############################################
#########   C O M M A N D  L I N E   #########
##############################################

my $REVISION = 0;
my $HELP = 0;
my $VERBOSE = 0;
my $EDIR_BASEDIR = $EDIR_BASEDIR_DEF;
my $EDIR_BOOK = $EDIR_BOOK_DEF;
my $EDIR_YEAR = $EDIR_YEAR_DEF;

GetOptions(
	"version" => \$REVISION,
	"help" => \$HELP,
	"verbose" => \$VERBOSE,
	"altroot=s" => \$ALTROOT,
	"altos=s" => \$ALTOS,
	"edir-basedir|basedir=s" => \$EDIR_BASEDIR,
	"edir-book|book=s" => \$EDIR_BOOK,
	"edir-year|year=s" => \$EDIR_YEAR,
) || die "Error: invalid command line option, exiting...\n";

#
# Global values which depend on command line options.
#
my $EDIR_DATEDIR         = $EDIR_BASEDIR . '/' . $EDIR_YEAR . $EDIR_BOOK;


# --version
if ($REVISION) {
    print("$PROGNAME $CVS_REVISION\n");
    exit($EXIT_OK);
}

# --help
if ($HELP) {
    print("$PROGNAME $CVS_REVISION\n");
    usage();
    exit($EXIT_OK);
}

#
# Don't accept any command line args... usage() and exit.
#
if ($#ARGV >= 0) {
    usage();
    exit($EXIT_COMMAND_LINE);
}

#
# kick off the application.
#
my $exit_status = main();

#
# Done - inform system what happened.
#
exit($exit_status);


###############################
#########   M A I N   #########
###############################

sub main
{
    my $rc = $EXIT_OK;

    loginfo("$PROGNAME $CVS_REVISION");

    #
    # Gotta be root or exit.
    #
    if ($EUID != 0) {
	usage();
	logerror("Error: You need root privileges to run this script");
	return($EXIT_MUST_BE_ROOT);
    }

    if (-d $EDIR_DATEDIR) {
	loginfo("uninstalling offcycle edir update package: $EDIR_DATEDIR");
	$rc = edir_uninstall();
	if ($rc == $EXIT_OK) {
	    loginfo("offcycle edir update package uninstalled");
	}
	else {
	    logerror("uninstall of offcycle edir update package returned status: $rc");
	}
    }
    else {
	logerror("offcycle edir update package does not exist: $EDIR_DATEDIR");
    }

    return($rc);
}


#############################################
#########   A P P L I C A T I O N   #########
#############################################

sub edir_uninstall
{
    my $rc = $EXIT_OK;

    my @edir_datedir = (
	"$EDIR_DATEDIR",
    );
    my @edir_scripts = (
	"$EDIR_INSTALLBASE_SCRIPT_DEF",
	"$EDIR_UPDATE_SCRIPT_DEF",
	"$EDIR_REVERT_SCRIPT_DEF",
    );
    my @edir_logs = (
	"$EDIR_INSTALLBASE_LOGFILE_DEF",
	"$EDIR_UPDATE_LOGFILE_DEF",
	"$EDIR_REVERT_LOGFILE_DEF",
	"$EDIR_SUMMARY_LOGFILE_DEF",
    );
    my @edir_configs = (
	"$EDIR_UPDATE_CONF_DEF",
	"$EDIR_LOCKFILE_DEF",
	"$EDIR_PIDFILE_DEF",
    );

    my @paths = (@edir_datedir, @edir_scripts, @edir_logs, @edir_configs);

    foreach my $path (@paths) {
	eu_rmit($path);
    }

    # cron job
    eu_rmcronjob($EDIR_CRON_JOB_FILE_DEF);

    return($rc);
}


sub eu_rmit
{
    my ($path) = @_;

    if (-e $path) {
	my $rm_opt = $EMPTY;
	if (-d $path) {
	    $rm_opt = "-rf";
	}
	system("rm $rm_opt $path");
    }
}


sub eu_rmcronjob
{
    my ($job_path) = @_;

    if (-e $job_path) {
	system("sudo rm $job_path");
    }
}


#########################################
#########   F U N C T I O N S   #########
#########################################

sub usage
{
    print("Usage:\n");
    print("$PROGNAME --version\n");
    print("$PROGNAME --help\n");
    print("$PROGNAME [--verbose]\n");
    print "$PROGNAME [options]\n";
    print("$PROGNAME\n");
    print "where options are:\n";
    print "\t--edir-basedir=s             path to top of installation tree\n";
    print "\t--edir-book=s                name of book (jan, may, sep)\n";
    print "\t--edir-year=s                year of book (YYYY)\n";
    print("\n");

    return(1);
}


sub loginfo
{
    my ($message) = @_;

    chomp($message);
    $message = "<I> $message";
    logit($message);

    return(1);
}


sub logerror
{
    my ($message) = @_;

    chomp($message);
    $message = "<E> $message";
    logit($message);

    return(1);
}


sub logit
{
    my ($message) = @_;

    my $logtime = strftime("%Y-%m-%d %H:%M:%S", localtime());

    chomp($message);
    if (open(my $logfh, '>>', $LOGFILE)) {
	print($logfh "$logtime ($PROGNAME-$PID) $message\n");
	close($logfh);
    }
    else {
	print "$logtime ($PROGNAME-$PID) $message\n";
    }

    if ($VERBOSE) {
	print $message . "\n";
    }

    return(1);
}


#################################################
#########   D O C U M E N T A T I O N   #########
#################################################

__END__

=pod

=head2 NAME

I<edir_uninstall> - uninstall the offcycle edir update package


=head2 VERSION

This documentation refers to version: $Revision: 1.1 $


=head2 OPTIONS

=over 4

=item B<--version>

Output the version number of the script and exit.

=item B<--help>

Output a short help message and exit.

=item B<--verbose>

Modifier: report extra information.

=item B<--edir-basedir>

Specify an alternative top level directory for offcycle edir update files.

=item B<--edir-book>

Specify the "book" of the offcycle edir update.

=item B<--edir-year>

Specify the "year" of the offcycle edir update.

=back


=head2 DESCRIPTION

The I<edir_uninstall.pl> script may be used to completely uninstall
the offcycle edir package.

The steps taken to uninstall the package are:

=over 4

=item 1.

Remove the following scripts:

 /d/daisy/bin/edir_installbase.pl
 /d/daisy/bin/edir_update.pl
 /d/daisy/bin/edir_revert.pl

=item 2.

Remove the following log files:

 /d/daisy/log/edir_installbase.log
 /d/daisy/log/edir_update.log
 /d/daisy/log/edir_revert.log
 /d/daisy/log/Delta_Stats.log

=item 3.

Remove the following config files:

 /d/daisy/config/edir_update.conf
 /d/daisy/config/edir_update.lock
 /d/daisy/config/edir_update.pid

=item 4.

Remove the cron job file:

 /etc/cron.d/edir_update

=back

=head2 DIAGNOSTICS

=over 4

=item Exit status 0 ($EXIT_OK)

Successful completion.

=item Exit status 1 ($EXIT_COMMAND_LINE)

In general, there was an issue with the syntax of the command line.

=item Exit status 2 ($EXIT_MUST_BE_ROOT)

The user must be root or running under "sudo".

=back


=head1 SEE ALSO

I<edir_installbase.pl>, I<edir_update.pl>, I<edir_revert.pl>


=cut
